useDynLib(caRamel, .registration=TRUE)
importFrom("graphics", "legend", "par", "plot", "points")
importFrom("stats", "rnorm", "runif", "sd")
importFrom("utils", "setTxtProgressBar", "txtProgressBar", "write.table")
import(geometry)
import(parallel)
export(Cextrap)
export(Cinterp)
export(Crecombination)
export(Cusecovar)
export(Dimprove)
export(boxes)
export(caRamel)
export(decrease_pop)
export(dominate)
export(dominated)
export(downsize)
export(matvcov)
export(newXval)
export(pareto)
export(rselect)
export(val2rank)
export(vol_splx)
export(plot_population)
export(plot_caramel)
export(plot_pareto)
