% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dominate.R
\name{dominate}
\alias{dominate}
\title{Successive Pareto fronts of a population}
\usage{
dominate(matobj)
}
\arguments{
\item{matobj}{: matrix [ NInd , NObj ] of objectives}
}
\value{
f : vector of dimension NInd of dominances
}
\description{
calculates the successive Pareto fronts of a population (classification "onion peel"), when objectives need to be maximized.
}
\examples{
# Definition of the parameters
matobj <- matrix(runif(200), 100, 2)
# Call the function
pareto_rank <- dominate(matobj)

}
\author{
Alban de Lavenne, Fabrice Zaoui
}
